/*
 * File:   IOCNFG.c
 * Author: iwamoto
 *
 * Created on November 18, 2020, 9:40 AM
 */

#include <xc.h>

// CONFIG
#pragma config FOSC = INTOSC, PWRTE = OFF, MCLRE = ON, CP = OFF
#pragma config BOREN = ON, CLKOUTEN = OFF, IESO = ON, FCMEN = ON
#pragma config WRT = OFF, PPS1WAY = ON, ZCD = OFF, PLLEN = ON
#pragma config STVREN = ON, BORV = LO, LPBOR = OFF, LVP = ON
#pragma config WDTCPS = WDTCPS1F, WDTE = OFF, WDTCWS = WDTCWSSW, WDTCCS = SWC

void IO_Init(void){
    OSCCON = 0x70;      // PLL ON 8MHz_HF;
    BORCON = 0x00;      // disabled
    while(PLLR == 0);   // PLL 安定待ち

    LATA   = 0x00; LATB   = 0x00; LATC   = 0x00;
    TRISA  = 0x17; TRISB  = 0x70; TRISC  = 0xDF;
    ANSELC = 0x4F; ANSELB = 0xD0; ANSELA = 0x17;
    WPUB   = 0x00; WPUA   = 0x00; WPUC   = 0x80;
    OPTION_REGbits.nWPUEN = 0;
    ODCONA  = 0x00; ODCONB  = 0x00; ODCONC  = 0x00;
    SLRCONA = 0x37; SLRCONB = 0xF0; SLRCONC = 0xFF;
    INLVLA  = 0x3F; INLVLB  = 0xF0; INLVLC  = 0xFF;

    RXPPS      = 0x0D;   //RB5->EUSART:RX;
    CLCIN0PPS  = 0x17;   //RC7->CLC1:CLCIN0;
    RB7PPS     = 0x12;   //RB7->EUSART:TX;
    SMT1WINPPS = 0x14;   //RC4->SMT1:SMT1WIN;
    SMT1SIGPPS = 0x14;   //RC4->SMT1:SMT1SIG;
    RC5PPS     = 0x0E;   //RC5->PWM3:PWM3OUT;
    T4PPS      = 0x17;   //RC7->TMR4:T4IN;

    BAUD1CON = 0x08;    // EUSART 設定 ------------------
    RC1STA = 0x90;      // 8-bit no parity
    TX1STA = 0x24;      //  115200 @ 32MHz
    SPBRGL = 0x44; SPBRGH = 0x00;

    SMT1CON0 = 0x8B;    // SMT1 設定 --------------------
    SMT1CON1 = 0x43;    // MODE High and Low time;Repeat
    SMT1STAT = 0x00;    //
    SMT1CLK = 0x04;     // clock : MFINTOSC 31.25kHz;
    SMT1WIN = 0x00;     // signal : CLC1OUT 反転入力
    SMT1SIG = 0x07;     // SMT PR : 0x1E84 (2 sec)
    SMT1PRU = 0x00; SMT1PRH = 0x1E; SMT1PRL = 0x84;

    PWM3CON = 0x90;     // PWM3 設定　---------------------
    PWM3DCH = 0x63;     // 反転出力  DC ; 499
    PWM3DCL = 0xC0;     // 連携 : TMR2
    CCPTMRSbits.P3TSEL = 0;

    CLC1POL = 0x82;     // CLC1 設定 ---------------------
    CLC1SEL0 = 0x1A;    // 入力 0 : T4_postscaled_out
    CLC1SEL1 = 0x00;    // 入力 1 : CLCIN0(RC7)
    CLC1SEL2 = 0x00;    // 入力 2、入力 3 使用しない
    CLC1SEL3 = 0x00;    //
    CLC1GLS0 = 0x02;    // Gate 0 :入力 0 を出力  clockに接続
    CLC1GLS1 = 0x08;    // Gate 1 :入力 1 を反転  D端子に接続
    CLC1GLS2 = 0x00;    // Gate 2 : 常に 0 出力
    CLC1GLS3 = 0x00;    // Gate 3 : 常に 0 出力
    CLC1CON = 0x84;     // Mode : D flip-flop : 反転出力

    // Set TMR4MFINTOSC 31.25khz;
    T4CLKCON = 0x05;        // TMR4 設定 ----------------
    T4HLT = 0x13;           // RC7の両エッジでカウントスタート
    T4RST = 0x00;           // MFINTOSC 31.25khzを1/4(128us)
    T4PR = 0x9B;            // 156カウント(20ms)で、PRと一致し、pulse出力
    T4TMR = 0x00;           // この動作をを繰り返す
    PIR2bits.TMR4IF = 0;    // クリア IF flag.
    T4CON = 0xA0;           // TMR4ON on;

    T2CLKCON = 0x00;        // TMR2 設定 -------------------
    T2HLT = 0x00;           // PWM3と連携しサイドトーン作成
    T2RST = 0x00;           // clock:Fosc/4 pre:1/1 post:1/5
    T2PR = 0x63;            // PR 99;
    T2TMR = 0x00;           // Mode : rollover
    PIR1bits.TMR2IF = 0;    // クリア IF flag
    PIE1bits.TMR2IE = 1;    // TMR2 割り込み有効
    T2CON = 0x84;           // TMR2ON on;
}

void putch(char txData){
   while(0 == PIR1bits.TXIF);
   TX1REG = txData;           // USARTに書き込む
}
